% UFget: MATLAB interface for the UF Sparse Matrix Collection.
%
%   UFget             - loads a matrix from the UF Sparse Matrix Collection.
%   UFweb             - opens the URL for a matrix.
%   UFgrep            - search for matrices in the UF Sparse Matrix Collection.
%   UFkinds           - get 'kind' of each problem in the UF collection.
%   UFsvd             - singular values of a matrix in the UF collection.
%
% Helper functions:
%
%   UFget_defaults    - returns default parameter settings for UFget.
%   UFget_example     - a demo for UFget.
%   UFget_lookup      - gets the group, name, and id of a matrix.
%
% Example:
%   help UFget

% Copyright 2012, Timothy A. Davis
