% DIMACS10
%
% Requires UFget, the MATLAB interface to the UF Sparse Matrix Collection
%
%   dimacs10         - returns a graph from the DIMACS10 test set, via the UF Collection.
%   dimacs10_demo    - reads in all 138 DIMACS10 graphs.
%   dimacs10_install - compiles and installs dimacs10 for use in MATLAB.
%   metis_graph_read - reads a graph file in METIS format into a MATLAB sparse matrix.
